#ifndef _Recorder_h
#include "Recorder.h"
#endif

#ifndef _ScopeView_h
#include "ScopeView.h"
#endif

#ifndef _Preferences_h
#include "Preferences.h"
#endif

#ifndef _BeNetWindow_h
#include "BeNetWindow.h"
#endif

#ifndef _Encode_h
#include "Encode.h"
#endif

#ifndef _Network
#include "network.h"
#endif

#ifndef _NetworkInterface_h
#include "NetworkInterface.h"
#endif

#include <stdio.h>


ScopeView* m_scopeView;
Encode encodeur;
float m_recVolume;

Recorder::Recorder(ScopeView* scopeView)
{
	m_scopeView = scopeView;
	m_roster = 0;
	m_recordNode = 0;
	m_recVolume = 1.0;
	
	SetRecVolume(((float)Preferences::Instance()->RecordVolume()/(float)100));

	status_t error;
	
	//	Make sure the media roster is there (which means the server is there).
	m_roster = BMediaRoster::Roster(&error);
	if (!m_roster)
		printf("error: BMediaRoster failed to initialise\n");

	if (m_roster->GetAudioInput(&m_audioInputNode) < B_OK)
		printf("error: No audio input found\n");

	//	Create our internal Node which records sound, and register it.
	m_recordNode = new SoundConsumer("Sound Recorder");
	if (m_roster->RegisterNode(m_recordNode) < B_OK)
	{
		printf("error: SoundConsumer not registered\n");
		if (m_recordNode)
			m_recordNode->Release();
	}
}

Recorder::~Recorder()
{
	//	The sound consumer and producer are Nodes; it has to be Release()d and the Roster
	//	will reap it when it's done.
	if (m_recordNode)
	{
		m_roster->UnregisterNode(m_recordNode);
		m_recordNode->Release();
	}
}


void
Recorder::Record()
{
	//	Hook up input
	status_t err = MakeRecordConnection(m_audioInputNode);
	if (err < B_OK) {
		printf("connect to the selected sound input\n");
		return;
	}

	//	And get it going...
	bigtime_t then = m_recordNode->TimeSource()->Now()+50000LL;
	m_roster->StartNode(m_recordNode->Node(), then);
	if (m_audioInputNode.kind & B_TIME_SOURCE) {
		m_roster->StartNode(m_audioInputNode, m_recordNode->TimeSource()->RealTimeFor(then, 0));
	}
	else {
		m_roster->StartNode(m_audioInputNode, then);
	}
}

status_t
Recorder::MakeRecordConnection(
	const media_node & input)
{
	//	Find an available output for the given input node.
	int32 count = 0;
	status_t err = m_roster->GetFreeOutputsFor(input, &m_audioOutput, 1, &count, B_MEDIA_RAW_AUDIO);
	if (err < B_OK) {
		printf("Recorder::MakeRecordConnection(): couldn't get free outputs from audio input node\n");
		return err;
	}
	if (count < 1) {
		printf("Recorder::MakeRecordConnection(): no free outputs from audio input node\n");
		return B_BUSY;
	}

	//	Find an available input for our own Node. Note that we go through the
	//	MediaRoster; calling Media Kit methods directly on Nodes in our app is
	//	not OK (because synchronization happens in the service thread, not in
	//	the calling thread).
	// TODO: explain this
	err = m_roster->GetFreeInputsFor(m_recordNode->Node(), &m_recInput, 1, &count, B_MEDIA_RAW_AUDIO);
	if (err < B_OK) {
		printf("Recorder::MakeRecordConnection(): couldn't get free inputs for sound recorder\n");
		return err;
	}
	if (count < 1) {
		printf("Recorder::MakeRecordConnection(): no free inputs for sound recorder\n");
		return B_BUSY;
	}

	//	Find out what the time source of the input is.
	//	For most nodes, we just use the preferred time source (the DAC) for synchronization.
	//	However, nodes that record from an input need to synchronize to the audio input node
	//	instead for best results.
	//	MakeTimeSourceFor gives us a "clone" of the time source node that we can manipulate
	//	to our heart's content. When we're done with it, though, we need to call Release()
	//	on the time source node, so that it keeps an accurate reference count and can delete
	//	itself when it's no longer needed.
	// TODO: what about filters connected to audio input?
	media_node use_time_source;
	BTimeSource * tsobj = m_roster->MakeTimeSourceFor(input);
	if (! tsobj) {
		printf("Recorder::MakeRecordConnection(): couldn't clone time source from audio input node\n");
		return B_MEDIA_BAD_NODE;
	}

	//	Apply the time source in effect to our own Node.
	err = m_roster->SetTimeSourceFor(m_recordNode->Node().node, tsobj->Node().node);
	if (err < B_OK) {
		printf("Recorder::MakeRecordConnection(): couldn't set the sound recorder's time source\n");
		tsobj->Release();
		return err;
	}

	//	Get a format, any format.
	media_format fmt;
	fmt.u.raw_audio = m_audioOutput.format.u.raw_audio;
	fmt.type = B_MEDIA_RAW_AUDIO;

	//	Tell the consumer where we want data to go.
	err = m_recordNode->SetHooks(DataEncode, NotifyDataEncode, this);
	if (err < B_OK) {
		printf("Recorder::MakeRecordConnection(): couldn't set the sound recorder's hook functions\n");
		tsobj->Release();
		return err;
	}

	//	Using the same structs for input and output is OK in BMediaRoster::Connect().
	err = m_roster->Connect(m_audioOutput.source, m_recInput.destination, &fmt, &m_audioOutput, &m_recInput);
	if (err < B_OK) {
		printf("Recorder::MakeRecordConnection(): failed to connect sound recorder to audio input node.\n");
		tsobj->Release();
		m_recordNode->SetHooks(0, 0, 0);
		return err;
	}

	//	Start the time source if it's not running.
	if ((tsobj->Node() != input) && !tsobj->IsRunning()) {
		m_roster->StartNode(tsobj->Node(), BTimeSource::RealTime());
	}
	tsobj->Release();	//	we're done with this time source instance!
	return B_OK;
}

status_t
Recorder::BreakRecordConnection()
{
	status_t err;

	//	If we are the last connection, the Node will stop automatically since it
	//	has nowhere to send data to.
	err = m_roster->StopNode(m_recInput.node, 0);
	err = m_roster->Disconnect(m_audioOutput.node.node, m_audioOutput.source, m_recInput.node.node, m_recInput.destination);
	m_audioOutput.source = media_source::null;
	m_recInput.destination = media_destination::null;
	return err;
}

status_t
Recorder::StopRecording()
{
	m_scopeView->Mute();

	BreakRecordConnection();
	m_recordNode->SetHooks(0, 0, 0);

	m_roster->StopNode(m_recordNode->Node(), 0);
	m_roster->StartNode(m_audioInputNode, 0);
	
	return B_OK;
}

void
Recorder::DataEncode(void* cookie, bigtime_t timestamp, void* data, size_t size, const media_raw_audio_format& format)
{
	if (!m_scopeView->IsShrink())
		m_scopeView->NewData( (int16*)data );

	bool non_null_trame = true;
	int nbr_skip = 4;
	
	if(Preferences::Instance()->UseModemQuality())
		nbr_skip = 6;
		
	if(Preferences::Instance()->UseCodec())
	{
		if((non_null_trame = encodeur.decisionToEncode(data, size, nbr_skip, m_recVolume)) == true)
		{
			int size_of_buf;
			char *buffer = encodeur.createBuffer(size, size_of_buf, nbr_skip);
			encodeur.encodeBuffer(data, size, buffer, nbr_skip);
			data = (void*)buffer;
			size = size_of_buf;
			//NetworkInterface::Instance()->Send(data, size);
		}		
	}

	// tell the network to send the voice data
	TRtpData rtpdata;

	memcpy(rtpdata.data, data, size);
	rtpdata.timestamp = CNetwork::Instance()->GetTimeStamp();
	rtpdata.size = size;

	if (CNetwork::Instance()->IsConnectionReady())
	{
		if(Preferences::Instance()->UseCodec())
		{
			if(non_null_trame == true){
				if(Preferences::Instance()->UseModemQuality())
				{
					if(send_data(find_thread(NET_THREAD_SEND_RTP), NET_ID_MODEM, &rtpdata, sizeof(TRtpData)) != B_NO_ERROR)
						printf("FATAL ERROR BLEU SCREEN\n");
				}
				else
				{
					if(send_data(find_thread(NET_THREAD_SEND_RTP), NET_ID_CODEC, &rtpdata, sizeof(TRtpData)) != B_NO_ERROR)
						printf("FATAL ERROR BLEU SCREEN\n");
				}
			}
		}
		else
		{
			if(send_data(find_thread(NET_THREAD_SEND_RTP), NET_ID_RAW, &rtpdata, sizeof(TRtpData)) != B_NO_ERROR)
				printf("FATAL ERROR BLEU SCREEN\n");
		}
	}
}


void
Recorder::NotifyDataEncode(void * cookie, int32 code, ...)
{

}


void
Recorder::SetRecVolume(float volume)
{
	m_recVolume = volume;
}


