/* connection.cc */
#ifndef __CONNECTION_H_
#include "connection.h"
#endif

#ifndef _Output_h
#include "Output.h"
#endif


//****************************************************************************************
//	Procedure:	CConnection::CConnection
//	Fonction:	Constructeur
//****************************************************************************************
CConnection::CConnection()
{
	m_bConnectionReady = false;	

	socketSendRtpUdp = -1;
	socketRecvRtpUdp = -1;
	socketSendRtcpUdp = -1;
	socketRecvRtcpUdp = -1;

	//Address: Local RTP (port 2000)
	addressLocalRtpUdp.sin_family = AF_INET;
	addressLocalRtpUdp.sin_port = htons(PORT_RTP_UDP);
	memset(addressLocalRtpUdp.sin_zero, 0, sizeof(addressLocalRtpUdp.sin_zero));
	addressLocalRtpUdp.sin_addr.s_addr = GetLocalAddr();

	//Address: Local RTCP (port 2001)
	addressLocalRtcpUdp.sin_family = AF_INET;
	addressLocalRtcpUdp.sin_port = htons(PORT_RTCP_UDP);
	memset(addressLocalRtcpUdp.sin_zero, 0, sizeof(addressLocalRtcpUdp.sin_zero));
	addressLocalRtcpUdp.sin_addr.s_addr = GetLocalAddr();
}

//****************************************************************************************
//	Procedure:	CConnection::~CConnection
//	Fonction:	Destructeur
//****************************************************************************************	
CConnection::~CConnection()
{
	closesocket(socketSendRtpUdp);
	closesocket(socketRecvRtpUdp);
	closesocket(socketSendRtcpUdp);
	closesocket(socketRecvRtcpUdp);
}

//****************************************************************************************
void CConnection::SetRemoteAddress(ulong RemoteAddress)
{
	struct hostent* hostRemote;
	
	hostRemote = gethostbyaddr((const char *)&RemoteAddress, 4, AF_INET);

	//!!!Sweet bug!!! Declarer les remotes AVANT les locals //

	//Address: Remote RTP (port 2000)
	addressRemoteRtpUdp.sin_family = AF_INET;
	addressRemoteRtpUdp.sin_port = htons(PORT_RTP_UDP);
	memset(addressRemoteRtpUdp.sin_zero, 0, sizeof(addressRemoteRtpUdp.sin_zero));
	addressRemoteRtpUdp.sin_addr.s_addr = *(long *)hostRemote->h_addr;
	
	//Address: Remote RTCP (port 2001)
	addressRemoteRtcpUdp.sin_family = AF_INET;
	addressRemoteRtcpUdp.sin_port = htons(PORT_RTCP_UDP);
	memset(addressRemoteRtcpUdp.sin_zero, 0, sizeof(addressRemoteRtcpUdp.sin_zero));
	addressRemoteRtcpUdp.sin_addr.s_addr = *(long *)hostRemote->h_addr;	

	m_bConnectionReady = true;
}

//****************************************************************************************
//	Procedure:	CConnection::RtpUdpMode
//	Fonction:	
//****************************************************************************************	
void CConnection::RtpUdpMode(MODE mode)
{
	switch (mode)
	{
	case SEND:
		if (socketSendRtpUdp < 0)
		{
			if((socketSendRtpUdp = socket(AF_INET, SOCK_DGRAM, 0)) < B_OK)
				cout << "ERROR: Unable to create RTP UDP socket - Cconnection::RtpUdpMode (SEND)" << endl;

			if ( setsockopt(socketSendRtpUdp, SOL_SOCKET, SO_NONBLOCK, "1", 1) != B_OK )
				cout  << "ERROR: setsockopt - Cconnection::RtpUdpMode (SEND)" << endl;
		}
		break;

	case RECV:
		if (socketRecvRtpUdp < 0)
		{
			if((socketRecvRtpUdp = socket(AF_INET, SOCK_DGRAM, 0)) < B_OK)
				cout << "ERROR: Unable to create RTP UDP socket - Cconnection::RtpUdpMode(RECV)" << endl;

			if ( setsockopt(socketRecvRtpUdp, SOL_SOCKET, SO_NONBLOCK, "1", 1) != B_OK )
				cout  << "ERROR: setsockopt - Cconnection::RtpUdpMode (RECV)" << endl;

			if ((bind(socketRecvRtpUdp, (sockaddr *)&addressLocalRtpUdp, sizeof(sockaddr_in))) < 0)
				cout << "ERROR: Unable to bind RTP Udp socket recv - CConnection::RtpUdpMode (RECV)" << endl;	
		}
		break;
	};
}


//****************************************************************************************
//	Procedure:	CConnection::RtcpUdpMode
//	Fonction:	
//****************************************************************************************	
void CConnection::RtcpUdpMode(MODE mode)
{
	switch (mode)
	{
	case SEND:
		if (socketSendRtcpUdp < 0)
		{
			if((socketSendRtcpUdp = socket(AF_INET, SOCK_DGRAM, 0)) < B_OK)
				cout << "ERROR: Unable to create RTCP UDP socket - CConnection::RtcpUdpMode (SEND)" << endl;
		
			if ( setsockopt(socketSendRtcpUdp, SOL_SOCKET, SO_NONBLOCK, "1", 1) != B_OK )
				cout  << "ERROR: setsockopt - CConnection::RtcpUdpMode (SEND)" << endl;
		}
		break;
		
	case RECV:
		if (socketRecvRtcpUdp < 0)
		{
			if((socketRecvRtcpUdp = socket(AF_INET, SOCK_DGRAM, 0)) < B_OK)
				cout << "ERROR: Unable to create RTCP UDP socket - CConnection::RtcpUdpMode (RECV)" << endl;
		
			if ( setsockopt(socketRecvRtcpUdp, SOL_SOCKET, SO_NONBLOCK, "1", 1) != B_OK )
				cout  << "ERROR: setsockopt - CConnection::RtcpUdpMode (RECV)" << endl;
	
			if ((bind(socketRecvRtcpUdp, (sockaddr *)&addressLocalRtcpUdp, sizeof(sockaddr_in))) < 0)
				cout << "ERROR: Unable to bind RTCP Udp socket recv - CConnection::RtcpUdpMode (RECV)" << endl;	
		}
		break;
	};
}


//****************************************************************************************
//	Procedure:	CConnection::SendRtpUdp
//	Fonction:	Envoyer une RTP 
//****************************************************************************************	
void CConnection::SendRtpUdp(char *data, int packetType)
{
	switch(packetType)
	{
		case(NET_ID_RAW):
			if (sendto(socketSendRtpUdp, data, RTPHEADERLEN + RTPPLLEN_RAW, 0, (sockaddr *)&addressRemoteRtpUdp, sizeof(sockaddr_in)) < 0)
			{
				cout << "ERROR: Unable to send by Audio UDP some data - Cconnection::SendUdp()" << endl;
			}
			break;	
		case(NET_ID_CODEC):
			if (sendto(socketSendRtpUdp, data, RTPHEADERLEN + RTPPLLEN_CODEC, 0, (sockaddr *)&addressRemoteRtpUdp, sizeof(sockaddr_in)) < 0)
			{
				cout << "ERROR: Unable to send by Codec UDP some data - Cconnection::SendUdp()" << endl;
			}
			break;
		case(NET_ID_MODEM):
			if (sendto(socketSendRtpUdp, data, RTPHEADERLEN + RTPPLLEN_MODEM, 0, (sockaddr *)&addressRemoteRtpUdp, sizeof(sockaddr_in)) < 0)
			{
				cout << "ERROR: Unable to send by Modem UDP some data - Cconnection::SendUdp()" << endl;
			}
			break;
	}
}


//****************************************************************************************
//	Procedure:	CConnection::RecvRtpUdp
//	Fonction:	Recevoir une trame RTP
//****************************************************************************************	
int CConnection::RecvRtpUdp(char* pchBuffer, int nMaxLen)
{
	return recvfrom(socketRecvRtpUdp, pchBuffer, nMaxLen, 0, NULL, NULL);
}


//****************************************************************************************
//	Procedure:	CConnection::SendRtcpUdp
//	Fonction:	Envoyer une trame RTCP 
//****************************************************************************************	
void CConnection::SendRtcpUdp(char *data)
{
	if (sendto(socketSendRtcpUdp, data, RTCPDATAGRAMMELEN, 0, (sockaddr *)&addressRemoteRtcpUdp, sizeof(sockaddr_in)) < 0)
	{
		cout << "ERROR: Unable to send by UDP some data - Cconnection::SendUdp()" << endl;
	}
}


//****************************************************************************************
//	Procedure:	CConnection::RecvRtcpUdp
//	Fonction:	Recevoir une trame RTCP
//****************************************************************************************	
int CConnection::RecvRtcpUdp(char* pchBuffer, int nMaxLen, ulong* lRemoteAddress)
{
	sockaddr_in sinRemote;
	int lenAddr;
	int rv = recvfrom(socketRecvRtcpUdp, pchBuffer, nMaxLen, 0, (sockaddr*)&sinRemote, &lenAddr);

	*lRemoteAddress = (ulong)sinRemote.sin_addr.s_addr;

	return rv;	
}


//****************************************************************************************
//	Procedure:	CConnection::GetLocalAddr
//	Fonction:	
//****************************************************************************************	
long CConnection::GetLocalAddr()
{ 
	struct hostent* hostLocal;
	
	if (gethostname(localName, MAXHOSTNAMELEN) == 0)
	{ 
		cout << "ERROR: Unable to get host name - Cconnection::GetLocalAddr()" << endl;
		return -1; 
	} 
   
	if ((hostLocal = gethostbyname(localName)) == NULL)
	{ 
		cout << "ERROR: Unable to get host by name - Cconnection::GetLocalAddr()" << endl;
		return -1; 
	} 
   
	return *(long *)hostLocal->h_addr; 
}
