#ifndef _PreferencesViews_h
#include "PreferencesViews.h"
#endif

#ifndef _Preferences_h
#include "Preferences.h"
#endif

#ifndef _Def_h
#include "Def.h"
#endif

#ifndef _Output_h
#include "Output.h"
#endif

#include <stdlib.h>

/************************************/
/* General Preferences              */
/************************************/
GeneralPreferencesView::GeneralPreferencesView(BRect frame)
	: BView(frame, "General", B_FOLLOW_NONE, B_WILL_DRAW)
{
	SetViewColor(216,216,216);

	AddChild(m_shrinkOscilloscope = new BCheckBox(BRect(10,15,100,30), "Oscilloscope", "Oscilloscope", new BMessage(MSG_GENERAL_SHOW_OSCILLOSCOPE)));
	AddChild(m_shrinkSoundControls = new BCheckBox(BRect(10,31,100,46), "Sound Controls", "Sound Controls", new BMessage(MSG_GENERAL_SHOW_SOUNDCONTROLS)));
	AddChild(m_shrinkConnection = new BCheckBox(BRect(10,47,100,62), "Connection", "Connection", new BMessage(MSG_GENERAL_SHOW_CONNECTION)));
	AddChild(m_bookmarksWindow = new BCheckBox(BRect(10,63,100,78), "Bookmarks", "Bookmarks", new BMessage(MSG_GENERAL_SHOW_BOOKMARKS)));
	AddChild(m_outputWindow = new BCheckBox(BRect(10,79,100,94), "Output", "Output", new BMessage(MSG_GENERAL_SHOW_OUTPUT)));

	// Set the controls' values
	m_shrinkOscilloscope->SetValue((int32)Preferences::Instance()->ShowOscilloscope());
	m_shrinkSoundControls->SetValue((int32)Preferences::Instance()->ShowSoundControls());
	m_shrinkConnection->SetValue((int32)Preferences::Instance()->ShowConnection());
	m_bookmarksWindow->SetValue((int32)Preferences::Instance()->ShowBookmarks());
	m_outputWindow->SetValue((int32)Preferences::Instance()->ShowOutput());
}

void
GeneralPreferencesView::AllAttached()
{
	m_shrinkOscilloscope->SetTarget(this);
	m_shrinkSoundControls->SetTarget(this);
	m_shrinkConnection->SetTarget(this);
	m_bookmarksWindow->SetTarget(this);
	m_outputWindow->SetTarget(this);
}

void
GeneralPreferencesView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MSG_GENERAL_SHOW_OSCILLOSCOPE:
		Output::Instance()->UI("Preferences General: MSG_GENERAL_SHOW_OSCILLOSCOPE\n");
		Preferences::Instance()->ShowOscilloscope(m_shrinkOscilloscope->Value());
		break;
	case MSG_GENERAL_SHOW_SOUNDCONTROLS:
		Output::Instance()->UI("Preferences General: MSG_GENERAL_SHOW_SOUNDCONTROLS\n");
		Preferences::Instance()->ShowSoundControls(m_shrinkSoundControls->Value());
		break;
	case MSG_GENERAL_SHOW_CONNECTION:
		Output::Instance()->UI("Preferences General: MSG_GENERAL_SHOW_CONNECTION\n");
		Preferences::Instance()->ShowConnection(m_shrinkConnection->Value());
		break;
	case MSG_GENERAL_SHOW_BOOKMARKS:
		Output::Instance()->UI("Preferences General: MSG_GENERAL_SHOW_BOOKMARKS\n");
		Preferences::Instance()->ShowBookmarks(m_bookmarksWindow->Value());
		break;
	case MSG_GENERAL_SHOW_OUTPUT:
		Output::Instance()->UI("Preferences General: MSG_GENERAL_SHOW_OUTPUT\n");
		Preferences::Instance()->ShowOutput(m_outputWindow->Value());
		break;
	default:
		BView::MessageReceived(msg);
		break;
	}
}


/************************************/
/* Audio Preferences                */
/************************************/
AudioPreferencesView::AudioPreferencesView(BRect frame)
	: BView(frame, "Audio", B_FOLLOW_NONE, B_WILL_DRAW)
{
	SetViewColor(216,216,216);

	AddChild(m_useModemQuality = new BCheckBox(BRect(10,15,100,30), "Modem Quality", "Modem Quality", new BMessage(MSG_AUDIO_USE_MODEL_QUALITY)));
	AddChild(m_useCodec = new BCheckBox(BRect(10,31,100,46), "Use codec", "Use codec", new BMessage(MSG_AUDIO_USE_CODEC)));

	// Set the controls' values
	m_useCodec->SetValue((int32)Preferences::Instance()->UseCodec());
	m_useModemQuality->SetValue((int32)Preferences::Instance()->UseModemQuality());
	m_useModemQuality->SetEnabled((bool)(m_useCodec->Value() == B_CONTROL_ON));
}

void
AudioPreferencesView::AllAttached()
{
	m_useModemQuality->SetTarget(this);
	m_useCodec->SetTarget(this);
}

void
AudioPreferencesView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MSG_AUDIO_USE_MODEL_QUALITY:
		Output::Instance()->UI("Preferences Audio: MSG_AUDIO_USE_MODEL_QUALITY\n");
		Preferences::Instance()->UseModemQuality(m_useModemQuality->Value());
		break;
	case MSG_AUDIO_USE_CODEC:
		Output::Instance()->UI("Preferences Audio: MSG_AUDIO_USE_CODEC\n");
		Preferences::Instance()->UseCodec(m_useCodec->Value());
		m_useModemQuality->SetEnabled((bool)(m_useCodec->Value() == B_CONTROL_ON));
		break;
	default:
		BView::MessageReceived(msg);
		break;
	}
}


/************************************/
/* Network Preferences              */
/************************************/
NetworkPreferencesView::NetworkPreferencesView(BRect frame)
	: BView(frame, "Network", B_FOLLOW_NONE, B_WILL_DRAW)
{
	SetViewColor(216,216,216);

	AddChild(m_pGoOnlineOnStartup = new BCheckBox(BRect(10,15,140,30), "Online", "Go online on startup", new BMessage(MSG_NETWORK_GOONLINE)));
	AddChild(m_connectionTimeout = new BTextControl(BRect(10,35,220,50), "timeout", "Connection timeout (min)", "5"/*Preferences::Instance()->ConnectionTimeout()*/, new BMessage(MSG_NETWORK_CONNECTION_TIMEOUT)));
	m_connectionTimeout->SetDivider(140);
	AddChild(m_pServerIP = new BTextControl(BRect(10,55,220,70), "serverIP", "Server IP address", Preferences::Instance()->ServerIP(), new BMessage(MSG_NETWORK_SERVER_IP)));
	m_pServerIP->SetDivider(140);
	AddChild(m_pServerPort = new BTextControl(BRect(10,75,220,90), "serverPort", "Server Port", "3030"/*Preferences::Instance()->ServerPort()*/, new BMessage(MSG_NETWORK_SERVER_PORT)));
	m_pServerPort->SetDivider(140);

	// Set the controls' values
	m_pGoOnlineOnStartup->SetValue((int32)Preferences::Instance()->OnlineOnStartup());
}

void
NetworkPreferencesView::AllAttached()
{
	m_connectionTimeout->SetTarget(this);
	m_pGoOnlineOnStartup->SetTarget(this);
	m_pServerIP->SetTarget(this);
	m_pServerPort->SetTarget(this);
}

void
NetworkPreferencesView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MSG_NETWORK_GOONLINE:
		Output::Instance()->UI("Preferences Network: MSG_NETWORK_GOONLINE\n");
		Preferences::Instance()->OnlineOnStartup(m_pGoOnlineOnStartup->Value());
		break;
	case MSG_NETWORK_CONNECTION_TIMEOUT:
		Output::Instance()->UI("Preferences Network: MSG_NETWORK_CONNECTION_TIMEOUT\n");
		Preferences::Instance()->ConnectionTimeout(atoi(m_connectionTimeout->Text()));
		break;
	case MSG_NETWORK_SERVER_IP:
		Output::Instance()->UI("Preferences Network: MSG_NETWORK_SERVER_IP\n");
		Preferences::Instance()->ServerIP((char*)m_pServerIP->Text());
		break;
	case MSG_NETWORK_SERVER_PORT:
		Output::Instance()->UI("Preferences Network: MSG_NETWORK_SERVER_PORT\n");
		Preferences::Instance()->ServerPort(atoi(m_pServerPort->Text()));
		break;
	default:
		BView::MessageReceived(msg);
		break;
	}
}


/************************************/
/* Identity Preferences             */
/************************************/
IdentityPreferencesView::IdentityPreferencesView(BRect frame)
	: BView(frame, "Identity", B_FOLLOW_NONE, B_WILL_DRAW)
{
	SetViewColor(216,216,216);

	AddChild(m_pFirstName = new BTextControl(BRect(10,15,245,30), "firstname", "First name", Preferences::Instance()->FirstName(), new BMessage(MSG_IDENTITY_FIRSTNAME)));
	m_pFirstName->SetDivider(70);
	AddChild(m_pLastName = new BTextControl(BRect(10,35,245,50), "lastname", "Last name", Preferences::Instance()->LastName(), new BMessage(MSG_IDENTITY_LASTNAME)));
	m_pLastName->SetDivider(70);
	AddChild(m_pEmail = new BTextControl(BRect(10,55,245,70), "email", "E-mail", Preferences::Instance()->Email(), new BMessage(MSG_IDENTITY_EMAIL)));
	m_pEmail->SetDivider(70);
}

void
IdentityPreferencesView::AllAttached()
{
	m_pFirstName->SetTarget(this);
	m_pLastName->SetTarget(this);
	m_pEmail->SetTarget(this);
}

void
IdentityPreferencesView::MessageReceived(BMessage* msg)
{
	switch(msg->what)
	{
	case MSG_IDENTITY_FIRSTNAME:
		Output::Instance()->UI("Preferences Identity: MSG_IDENTITY_FIRSTNAME\n");
		Preferences::Instance()->FirstName((char*)m_pFirstName->Text());
		break;
	case MSG_IDENTITY_LASTNAME:
		Output::Instance()->UI("Preferences Identity: MSG_IDENTITY_LASTNAME\n");
		Preferences::Instance()->LastName((char*)m_pLastName->Text());
		break;
	case MSG_IDENTITY_EMAIL:
		Output::Instance()->UI("Preferences Identity: MSG_IDENTITY_EMAIL\n");
		Preferences::Instance()->Email((char*)m_pEmail->Text());
		break;
	default:
		BView::MessageReceived(msg);
		break;
	}
}
